//    Copyright (C) 2009  Regis COSNIER
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

using System.Drawing;
using System.Drawing.Imaging;
using OpenTK.Graphics;
using System;
using System.IO;

namespace StreetViewMapper
{
	// Sample
	//GL.Clear(ClearBufferMask.ColorBufferBit | ClearBufferMask.DepthBufferBit);
	//GL.MatrixMode(MatrixMode.Modelview);
	//GL.LoadIdentity();
	//Glu.LookAt(new Vector3(0.0f, 0.0f, 2.0f), -Vector3.UnitZ, Vector3.UnitY);

	//GL.Rotate(globalPhi, Vector3.UnitX);
	//GL.Rotate(globalTheta, Vector3.UnitY);

	//cubeMap.Begin();
	//int GL_NORMAL_MAP_EXT = 0x8511;
	//int GL_REFLECTION_MAP_EXT = 0x8512;
	//GL.TexGen(TextureCoordName.S, TextureGenParameter.TextureGenMode, GL_REFLECTION_MAP_EXT);
	//GL.TexGen(TextureCoordName.T, TextureGenParameter.TextureGenMode, GL_REFLECTION_MAP_EXT);
	//GL.TexGen(TextureCoordName.R, TextureGenParameter.TextureGenMode, GL_REFLECTION_MAP_EXT);
	//GL.Enable(EnableCap.TextureGenS);
	//GL.Enable(EnableCap.TextureGenT);
	//GL.Enable(EnableCap.TextureGenR);

	//GL.Enable(EnableCap.CullFace);
	//GL.CullFace(CullFaceMode.Back);

	////GL.Enable(EnableCap.Lighting);
	////GL.Enable(EnableCap.Light0);
	////GL.Lightv(LightName.Light0, LightParameter.Position, new Vector4(1.0f, 1.0f, -1.0f, 1.0f));

	////GL.Materialv(MaterialFace.Front, MaterialParameter.Ambient, new Color4(0.8f, 0.8f, 0.8f, 1.0f));
	////GL.Materialv(MaterialFace.Front, MaterialParameter.Diffuse, new Color4(1.0f, 1.0f, 1.0f, 1.0f));

	//GL.TexEnv(TextureEnvTarget.TextureEnv, TextureEnvParameter.TextureEnvMode, (int)TextureEnvMode.Modulate);

	//IntPtr quadratic = Glu.NewQuadric();
	//Glu.QuadricTexture(quadratic, true);
	//Glu.QuadricNormal(quadratic, QuadricNormal.Smooth);

	//Glu.Sphere(quadratic, 1.0, 20, 30);

	//GL.TexGen(TextureCoordName.S, TextureGenParameter.TextureGenMode, GL_NORMAL_MAP_EXT);
	//GL.TexGen(TextureCoordName.T, TextureGenParameter.TextureGenMode, GL_NORMAL_MAP_EXT);
	//GL.TexGen(TextureCoordName.R, TextureGenParameter.TextureGenMode, GL_NORMAL_MAP_EXT);
	//GL.Disable(EnableCap.TextureGenS);
	//GL.Disable(EnableCap.TextureGenT);
	//GL.Disable(EnableCap.TextureGenR);
	//cubeMap.End();

	public class CubeMap
	{
		int cubeTextureId = -1;

		public string Folder { get; set; }

		public void Init()
		{
			Release();

			string[] imageName = new string[] {
			    "posx.jpg", // GL_TEXTURE_CUBE_MAP_POSITIVE_X
			    "negx.jpg", // GL_TEXTURE_CUBE_MAP_NEGATIVE_X
			    "posy.jpg", // GL_TEXTURE_CUBE_MAP_POSITIVE_Y
			    "negy.jpg", // GL_TEXTURE_CUBE_MAP_NEGATIVE_Y
			    "posz.jpg", // GL_TEXTURE_CUBE_MAP_POSITIVE_Z
			    "negz.jpg"  // GL_TEXTURE_CUBE_MAP_NEGATIVE_Z
			};


			int[] cId = new int[1];
			GL.GenTextures(1, cId);
			this.cubeTextureId = cId[0];
			GL.Enable(EnableCap.TextureCubeMap);
			GL.BindTexture(TextureTarget.TextureCubeMap, this.cubeTextureId);
			GL.TexParameter(TextureTarget.TextureCubeMap, TextureParameterName.TextureMagFilter, (int)TextureMinFilter.Linear);
			GL.TexParameter(TextureTarget.TextureCubeMap, TextureParameterName.TextureMinFilter, (int)TextureMinFilter.Linear); //TextureMinFilter.LinearMipmapLinear); //GL._LINEAR); //GL._LINEAR_MIPMAP_LINEAR);
			GL.TexParameter(TextureTarget.TextureCubeMap, TextureParameterName.GenerateMipmap, (int)All.True);
			for (int i = 0; i < 6; i++)
			{
				Bitmap bitmap = null;
				try
				{
					bitmap = new Bitmap(Path.Combine(Folder, imageName[i]));

					BitmapData data = bitmap.LockBits(new Rectangle(0, 0, bitmap.Width, bitmap.Height),
						ImageLockMode.ReadOnly, System.Drawing.Imaging.PixelFormat.Format32bppArgb);
					GL.TexImage2D(TextureTarget.TextureCubeMapPositiveX + i, 0, PixelInternalFormat.Rgba, data.Width, data.Height, 0,
						OpenTK.Graphics.PixelFormat.Bgra, PixelType.UnsignedByte, data.Scan0);
					bitmap.UnlockBits(data);
				}
				catch (Exception ex)
				{
					Release();
					throw new Exception(string.Format("Cannot load the cubemap image ('{0}').\r\n{1}", imageName[i], ex.Message), ex);
				}
				finally
				{
					if (bitmap != null)
						bitmap.Dispose();
				}
			}

			GL.TexParameter(TextureTarget.TextureCubeMap, TextureParameterName.GenerateMipmap, (int)All.False);
			GL.TexParameter(TextureTarget.TextureCubeMap, TextureParameterName.TextureWrapS, (int)TextureWrapMode.ClampToEdge);
			GL.TexParameter(TextureTarget.TextureCubeMap, TextureParameterName.TextureWrapT, (int)TextureWrapMode.ClampToEdge);
			GL.TexParameter(TextureTarget.TextureCubeMap, TextureParameterName.TextureWrapR, (int)TextureWrapMode.ClampToEdge);
			GL.Disable(EnableCap.TextureCubeMap);
		}

		public void Release()
		{
			if (this.cubeTextureId > 0)
			{
				GL.DeleteTextures(1, new int[] { this.cubeTextureId });
				this.cubeTextureId = 0;
			}
		}

		public void Begin()
		{
			GL.Enable(EnableCap.TextureCubeMap);
			GL.BindTexture(TextureTarget.TextureCubeMap, cubeTextureId);
		}

		public void End()
		{
			GL.BindTexture(TextureTarget.TextureCubeMap, 0);
			GL.Disable(EnableCap.TextureCubeMap);
		}
	}
}
